﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Printing;
using System.Text;
using System.Windows.Forms;

// A4 11.5" x 8", @ 300 dpi = 3450 x 2400.

namespace BrickArch
{
    public partial class Bricks : Form
    {
        /* bond string format
         * <space in 1/8ths of stretcher> <brick pattern> ',' <next line space> <brick pattern> ','
         * 0-9 space in 1/8th of brick
         * C corner brick, header on current face, stretcher on next face
         * c corner brick, stretcher on current face, header on the next face
         * Q 3/4 brick, q quarter brick
         * S stretcher, H header (1/2 stretcher), 
         * A alternate colour stretcher, a alternate colour header
         * J sailor (3 row high, header width) , K soldier (1/3 stretcher width, 3 rows high)
         * V small (2 row) vertical brick, v bottom of V,
         * T small 2/3 stretcher, t small 1/3 stretcher
         * R shiner, r rowlock. Bricks on edge to make 4.5" row height as in Rat trap bond.
         * , end of course pattern, repeats from 2nd character
         */
        public class Bondc
        {
            public string name;
            public string pattern;
            public Bondc(string s1, string s2)
            {
                name = s1;
                pattern = s2;
            }
        };
        public string bond = "0S,4S,";
        public int rowsize = 12;
        public int bricklength = 36;
        public int brickwidth = 18;
        public int mortar = 2;
        public Color brickcolor = Color.FromArgb(255, 125, 40, 40);
        const int NumberOfBrickShades = 8;
        SolidBrush[] brush = new SolidBrush[NumberOfBrickShades];
        public Color altbrickcolor = Color.FromArgb(255, 70, 20, 20);
        public Color mortarcolor = Color.FromArgb(255, 250, 250, 235);
        const float pi = 3.1415927F;

        public const int pagewidth = 3450;
        public const int pageheight = 2400;
        public int pageorgx = 0;
        public int pageorgy = 0;
        public Bitmap page = new Bitmap(pagewidth, pageheight);
        public bool enabledraw = false;


        public Bondc[] bonds = {
            new Bondc("Stretcher or Running Bond", "0S,4S,"),
            new Bondc("Raking Stretcher Bond","0S,3S,6S,"),
            new Bondc("English or Ancient bond", "0S,2H,"),
            new Bondc("English Cross or Dutch bond", "0S,2H,4S,2H,"),
            new Bondc("English Garden Wall or Common bond", "0S,4S,0S,2H,"),
            new Bondc("American bond", "0S,4S,0S,4S,0S,2H,"),
            new Bondc("Flemish bond", "0SH,2HS,"),
            new Bondc("Flemish diagonal bond", "0SH,2S,"),
            new Bondc("Flemish double stretcher, Monk or Yorkshire bond", "0SHS,2SSH,"),
            new Bondc("Flemish double header bond", "0SHH,2HSH,"),
            new Bondc("Flemish garden wall or Wessex bond", "0SSHS,2HSSS,"),
            new Bondc("Header bond", "0H,2H,"),
            new Bondc("Stack bond", "0S,"),
            new Bondc("Herringbone","0KSvv,0vKSv,0vvKS,vvvKS,TvvKS,0SvvK,"),
            new Bondc("Basket", "0KKKS,0vvvS,0vvvS,0SKKK,0Svvv,0Svvv,"),
            new Bondc("Flemish cross","0SaSa,2HSHA,0SaSa,2HAHS,"),
            new Bondc("Diamond Diaper", "0HaHa,6aaS,0HHaH,6aaS,0HaHa,2aSa,4HHHa,2aSa,"),
            new Bondc("Flemish garden wall or Sussex bond", "4SASH,2SaAS,8AAHS,6AHAS,4ASAH,2ASHA,8SSaA,6SHSA,8SSAa,2AHSA,4ASAH,6AHAS,4HAAS,2SAaS,"),
            new Bondc("Small brick Herringbone", "0VTv,0vVT,vvVT,0TvV,"),
            new Bondc("Small brick Basket", "0VVT,0vvT,0TVV,0Tvv,"),
            new Bondc("Small brick Pinwheel", "0VT,0vtV,0Tv,"),
            new Bondc("Rat trap bond","0Rr,3rR,"),
            new Bondc("Pilaster 2 - 3 1/2","0SS2SHS2SSS2SqSqS2SHHHS2,0HqHqH2HqSqH2HqHHHqH2HSHSH2HqSSqH2,"),
            new Bondc("Pilaster 4 - 4 1/2","0SSSS2SSSS2SHSHS2SSHSS2SSHSS2SHHHHHS2,0HSSSH2HqHHHHHqH2HqSHSqH2HqSSSqH2HqHHSHHqH2HqSSSqH2,"),
            new Bondc("Pilaster 5 - 5 1/2","0SSSSS2SSSSS2SqSHSqS2SqSSSqS2SHSHSHS2,0HSSSSH2HqHHHHHHHqH2HSHSHSH2HSHHHHHSH2HqSHSHSqH2,"),
            new Bondc("Pilaster 6 - 6 1/2","0SSSSSS2SSSSSS2HSHSHSHSH2SSSHSSS2,0HSSSSSH2HqHHHHHHHHHqH2SqSHSHSqS2HqHHHHHHHHHHqH2,"),
        };
        
        public Bricks()
        {
            InitializeComponent();

            for (int i=0; i<bonds.Length; i++)
            {
                BondCB.Items.Add(bonds[i].name);
            }
            bond = bonds[0].pattern;
            PatternTB.Text = bonds[0].pattern;
            ColourTB.BackColor = brickcolor;
            ShadeTB.Text = "25";
            BrightTB.Text = "35";
            AltColourTB.BackColor = altbrickcolor;
            MortarTB.BackColor = mortarcolor;
            HeightTB.Text = "3";
            WidthTB.Text = "40";
            CoursesTB.Text = "3";
            WallRD.Checked = true;
            enabledraw = true;

            Drawbricks();
        }

        public void Drawbricks()
        {
            if (!enabledraw)
                return;
            if (CircleRB.Checked)
                DrawCircle();
            else if (WallRD.Checked)
                DrawWall();
            else 
                Draw3CentreArch();
        }

        public void MakeBrushes()
        {
            Random rnd = new Random();
            int variation = Convert.ToInt16(ShadeTB.Text);
            int brightness = Convert.ToInt16(BrightTB.Text);
            for (int i = 0; i < NumberOfBrickShades; i++)
            {
                int r, g, b;
                int t = rnd.Next(brightness) - brightness / 2;
                r = brickcolor.R + t;
                g = brickcolor.G + t;
                b = brickcolor.B + t;
                r += rnd.Next(variation) - variation / 2;
                g += rnd.Next(variation) - variation / 2;
                b += rnd.Next(variation) - variation / 2;
                if (r < 0) r = 0;
                if (r > 255) r = 255;
                if (g < 0) g = 0;
                if (g > 255) g = 255;
                if (b < 0) b = 0;
                if (b > 255) b = 255;
                brush[i] = new SolidBrush(Color.FromArgb(255, r, g, b));
            }
        }

        // type = 0 rowlock, = 1 soldier, = 2 rowlock + soldier, = 3 soldier + rowlock
        // width = brick + mortar width, usually 12 pixel
        // radius = radius of circle
        // angle = angle of the center of the brick
        // x, y = location of the center of the circle
        // b = brush to be used
        // surface = graphics surface to draw on
        public void DrawAngledBrick(int type, float width, float radius, float angle, float x, float y, Brush b, Graphics surface)
        {
            PointF[] points = new PointF[4];
            int length = 18-2;
            if (type == 1 || type == 3)
                length = 36 - 2;
            width = width / 2 - 1;
            // convert (x,y) centre of circle to centre of inner edge of brick
            x += (float)(radius * Math.Sin(angle));
            y += (float)(radius * Math.Cos(angle));
            points[0].X = x - (float)(width * Math.Cos(angle)); // corners
            points[0].Y = y + (float)(width * Math.Sin(angle));
            points[1].X = x + (float)(width * Math.Cos(angle));
            points[1].Y = y - (float)(width * Math.Sin(angle));
            // convert (x,y) to centre of outer edge of brick
            x += (float)(length * Math.Sin(angle));
            y += (float)(length * Math.Cos(angle));
            if (TaperCB.Checked)
                width = width * (radius+length) / radius;
            radius += length;
            points[2].X = x + (float)(width * Math.Cos(angle));
            points[2].Y = y - (float)(width * Math.Sin(angle));
            points[3].X = x - (float)(width * Math.Cos(angle));
            points[3].Y = y + (float)(width * Math.Sin(angle));
            surface.FillPolygon(b, points);

            if (type == 2 || type == 3) // bonded, draw the second brick
            {
                // second brick inner edge
                x += (float)(2 * Math.Sin(angle));
                y += (float)(2 * Math.Cos(angle));
                if (type == 2)
                    length = 36 - 2;
                else
                    length = 18 - 2;
                if (TaperCB.Checked)
                    width = width * (radius+2) / radius;
                radius += 2;
                points[0].X = x - (float)(width * Math.Cos(angle)); // corners
                points[0].Y = y + (float)(width * Math.Sin(angle));
                points[1].X = x + (float)(width * Math.Cos(angle));
                points[1].Y = y - (float)(width * Math.Sin(angle));
                // second brick outer edge
                x += (float)(length * Math.Sin(angle));
                y += (float)(length * Math.Cos(angle));
                if (TaperCB.Checked)
                    width = width * (radius+length) / radius;
                points[2].X = x + (float)(width * Math.Cos(angle));
                points[2].Y = y - (float)(width * Math.Sin(angle));
                points[3].X = x - (float)(width * Math.Cos(angle));
                points[3].Y = y + (float)(width * Math.Sin(angle));
                surface.FillPolygon(b, points);
            }
        }

        public void DrawCircle()
        {
            MakeBrushes();
            Random rnd = new Random();
            float radius = Convert.ToInt16(WidthTB.Text) / 2 * 4; // pixels
            RadiusTB.Text = Radius2TB.Text = (radius/4).ToString();
            Graphics surface = System.Drawing.Graphics.FromImage(page);
            surface.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            surface.FillRectangle(new SolidBrush(mortarcolor), 0, 0, pagewidth, pageheight);
            float xorg, yorg;
            int courses = 1;
            if (BondedRB.Checked)
            {
                xorg = radius + 10 + courses * (18 + 36);
                yorg = radius + 10 + courses * (18 + 36);
            }
            else
            {
                courses = Convert.ToInt16(CoursesTB.Text);
                if (RowlockBtn.Checked)
                {
                    xorg = radius + 10 + courses * 18;
                    yorg = radius + 10 + courses * 18;
                }
                else
                {
                    xorg = radius + 10 + courses * 36;
                    yorg = radius + 10 + courses * 36;
                }
            }
            PointF[] points = new PointF[4];

            for (int j = 0; j < courses; j++)
            {
                int no = (int)Math.Ceiling(radius * pi / 12);
                radius = (float)no * 12 / pi;
                float angle = pi / no; // angle for each brick in radians

                int type = 0;
                if (SoldierBtn.Checked)
                    type = 1;
                else if (BondedRB.Checked)
                    type = 2;

                for (int i = 0; i <= no * 2; i++)
                {
                    DrawAngledBrick(type, 12, radius, angle * i, xorg, yorg, brush[rnd.Next(NumberOfBrickShades)], surface);
                    if (BondedRB.Checked)
                        type = 5 - type;
                }
                if (RowlockBtn.Checked)
                    radius += 18;
                else
                    radius += 36;
            }
            panel1.Invalidate();
        }

        // 3 center arch or roman arch
        public void Draw3CentreArch()
        {
            float sqrt2 = (float)Math.Sqrt(2);
            MakeBrushes();
            Random rnd = new Random();
            Graphics surface = System.Drawing.Graphics.FromImage(page);
            surface.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            surface.FillRectangle(new SolidBrush(mortarcolor), 0, 0, pagewidth, pageheight);

            float xorg1, yorg1, xorg2, yorg2, xorg3;
            float radius1, radius2;
            int size, sizex, sizey;
            int type = 0;
            int courses = Convert.ToInt16(CoursesTB.Text);
            float width = (float)Convert.ToDouble(WidthTB.Text) * 4;
            float height = (float)Convert.ToDouble(HeightTB.Text) * 4;

            if (BondedRB.Checked)
            {
                courses = 1;
                size = (18 + 36);
                type = 2;
            }
            else if (RowlockBtn.Checked)
            {
                courses = Convert.ToInt16(CoursesTB.Text);
                size = 18 * courses;
                type = 0;
            }
            else // soldier
            {
                courses = 1;
                size = 36;
                type = 1;
            }
            
            if (ArchRB.Checked) // segmental arch
            {
                if (height < 1)
                    height = 1;
                if (height > width / 2)
                    height = width / 2;
                radius1 = radius2 = (height * height + width * width / 4) / 2 / height; // pixels
                xorg1 = xorg2 = xorg3 = width/2 + 10 + size;
                yorg1 = yorg2 = radius1 + 10 + size;
            }
            else if (RomanRB.Checked) // roman
            {
                height = width / 2;
                radius1 = radius2 = height = width / 2;
                xorg1 = xorg2 = xorg3 = radius1 + 10 + size;
                yorg1 = yorg2 = radius1 + 10 + size;
            }
            else // 3 centre arch
            {
                if (height < width * 0.3)
                    height = (float)(width * 0.3);
                if (height > width)
                    height = width;
                float r3 = (width / 2 - height) / ( 2 - sqrt2);
                radius1 = width / 2 - r3;
                radius2 = r3 + height;
                xorg1 = radius1 + 10 + size;
                xorg3 = xorg1 + 2 * r3;
                yorg1 = height + 10 + size;
                xorg2 = xorg1 + r3;
                yorg2 = yorg1 + r3;
            }
            HeightTB.Text = (height / 4).ToString("F");
            RadiusTB.Text = (radius1 / 4).ToString("F");
            Radius2TB.Text = (radius2 / 4).ToString("F");

            sizex = (int)(width + 20 + 2 * size);
            sizey = (int)(height + 20 + size);

            float xorgstart1 = xorg1;
            float xorgstart2 = xorg2;
            float xorgstart3 = xorg3;
            float radiusstart1 = radius1;
            float radiusstart2 = radius2;
            float startangle = 0;

            if (ArchRB.Checked)
            {
                startangle = (float)Math.Acos(width / 2 / radius1);
            }

            for (int rows = 0; rows < 8; rows++)
             {
                 xorg1 = xorgstart1;
                 xorg2 = xorgstart2;
                 xorg3 = xorgstart3;
                 for (int cols = 0; cols < 5; cols++)
                 {
                     radius1 = radiusstart1;
                     radius2 = radiusstart2;
                     for (int i = 0; i < courses; i++)
                     {
                         float ba = (float)Math.Asin(12 / radius1);
                         float angle = startangle + ba / 2;
                         while (angle < pi / 4)
                         {
                             DrawAngledBrick(type, 12, radius1, -pi / 2 - angle, xorg1, yorg1, brush[rnd.Next(NumberOfBrickShades)], surface);
                             DrawAngledBrick(type, 12, radius1, +pi / 2 + angle, xorg3, yorg1, brush[rnd.Next(NumberOfBrickShades)], surface);
                             if (BondedRB.Checked)
                                 type = 5 - type;
                             angle += ba;
                         }

                         float t = angle - pi / 4;
                         t = (float)(radius1 * Math.Sin(t));
                         t = (float)Math.Asin(t / radius2);
                         angle = pi / 4 + t;

                         ba = (float)Math.Asin(12 / radius2);
                         while (true)
                         {
                             float bw = 12;
                             angle -= ba / 2;
                             float d = (float)(radius2 * Math.Sin(pi / 2 - angle) * 2);
                             if (d <= 2) // only mortar
                                 break;
                             else if (d <= 13) // one brick or less
                             {
                                 DrawAngledBrick(type, d, radius2, pi, xorg2, yorg2, brush[rnd.Next(NumberOfBrickShades)], surface);
                                 break;
                             }
                             else if (d < 72) // 6 bricks or less
                             {
                                 int n = (int)Math.Ceiling(d / 12);
                                 bw = d / n;
                             }
                             else // morethan 4 bricks
                                 bw = 12;
                             ba = (float)Math.Asin(bw / radius2);
                             angle += ba / 2;
                             DrawAngledBrick(type, bw, radius2, -pi / 2 - angle, xorg2, yorg2, brush[rnd.Next(NumberOfBrickShades)], surface);
                             DrawAngledBrick(type, bw, radius2, +pi / 2 + angle, xorg2, yorg2, brush[rnd.Next(NumberOfBrickShades)], surface);
                             if (BondedRB.Checked)
                                 type = 5 - type;
                             angle += ba;
                         }
                         radius1 += 18;
                         radius2 += 18;
                     }
                     xorg1 += sizex;
                     xorg2 += sizex;
                     xorg3 += sizex;
                 }
                 yorg1 += sizey;
                 yorg2 += sizey;
             }
            panel1.Invalidate();    
        }
        
        public void DrawWall()
        {
            MakeBrushes();
            Random rnd = new Random();
            SolidBrush altbrush = new SolidBrush(altbrickcolor);

            // check pattern ends in ,
            if (bond[bond.Length - 1] != ',')
                bond += ',';

            int x = 0, y = 0;
            int bondindex = 0;
            int rowstart = 0;
            int rowend = -1;

            Graphics surface = System.Drawing.Graphics.FromImage(page);
            surface.FillRectangle(new SolidBrush(mortarcolor), 0, 0, pagewidth, pageheight);

            while (y < pageheight)
            {
                rowstart = rowend + 1;
                if (rowstart >= bond.Length) // start from begining again
                    rowstart = 0;
                bondindex = rowstart;
                int rowheight = rowsize;
                while (x < pagewidth)
                {
                    switch(bond[bondindex]) {
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                            {
                                int t = (bond[bondindex] - '0') * bricklength / 8;
                                x += t;
                                bondindex++;
                            }
                            break;
                        case 'C': // corner stretcher
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, brickwidth - mortar, rowsize - mortar);
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x + brickwidth - mortar, y, bricklength - mortar, rowsize - mortar);
                            x += bricklength + brickwidth - mortar;
                            bondindex++;
                            break;
                        case 'c': // corner header
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, bricklength - mortar, rowsize - mortar);
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x + bricklength - mortar, y, brickwidth - mortar, rowsize - mortar);
                            x += brickwidth + bricklength - mortar;
                            bondindex++;
                            break;
                        case 'S': // stretcher
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, bricklength - mortar, rowsize - mortar);
                            x += bricklength;
                            bondindex++;
                            break;
                        case 'H': // header
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, brickwidth - mortar, rowsize - mortar);
                            x += brickwidth;
                            bondindex++;
                            break;
                        case 'Q': // 3/4 stretcher
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, 3 * brickwidth / 2 - mortar, rowsize - mortar);
                            x += 3 * brickwidth / 2;
                            bondindex++;
                            break;
                        case 'q': // 1/4 stretcher, 1/2 header, queen closer ?
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, brickwidth / 2 - mortar, rowsize - mortar);
                            x += brickwidth / 2;
                            bondindex++;
                            break;
                        case 'J': // Sailor
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, brickwidth - mortar, rowsize * 3 - mortar);
                            x += brickwidth;
                            bondindex++;
                            break;
                        case 'K': // Soldier
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, rowsize - mortar, rowsize * 3 - mortar);
                            x += rowsize;
                            bondindex++;
                            break;
                        case 'A': // alternate stretcher
                            surface.FillRectangle(altbrush, x, y, bricklength - mortar, rowsize - mortar);
                            x += bricklength;
                            bondindex++;
                            break;
                        case 'a': // alternate header
                            surface.FillRectangle(altbrush, x, y, brickwidth - mortar, rowsize - mortar);
                            x += brickwidth;
                            bondindex++;
                            break;
                        case 'V': // small brick soldier
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, rowsize - mortar, rowsize * 2 - mortar);
                            x += rowsize;
                            bondindex++;
                            break;
                        case 'v': // invisible small brick header
                            x += rowsize;
                            bondindex++;
                            break;
                        case 'T': // small brick stretcher
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, rowsize * 2 - mortar, rowsize - mortar);
                            x += rowsize * 2;
                            bondindex++;
                            break;
                        case 't': // small brick header
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, rowsize - mortar, rowsize - mortar);
                            x += rowsize;
                            bondindex++;
                            break;
                        case 'R': // Shiner
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, bricklength - mortar, brickwidth - mortar);
                            x += bricklength;
                            rowheight = brickwidth;
                            bondindex++;
                            break;
                        case 'r': // Rowlock
                            surface.FillRectangle(brush[rnd.Next(NumberOfBrickShades)], x, y, rowsize - mortar, brickwidth - mortar);
                            x += rowsize;
                            rowheight = brickwidth;
                            bondindex++;
                            break;
                        case ',': // row pattern end
                            rowend = bondindex;
                            bondindex = rowstart + 1;
                            break;
                        default:
                            panel1.Invalidate();
                            MessageBox.Show("Character '"+bond[bondindex]+"' is not allowed in a brick pattern","Brick pattern error");
                            return;
                    }
                }
                y += rowheight;
                x = 0;
            }
            panel1.Invalidate();    
        }

        private void panel_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawImage(page, -pageorgx, -pageorgy);
        }

        private void BondCB_SelectedIndexChanged(object sender, EventArgs e)
        {
            string t = BondCB.Text;
            for (int i = 0; i < bonds.Length; i++)
            {
                if (t == bonds[i].name)
                {
                    bond = bonds[i].pattern;
                    PatternTB.Text = bond;
                    Bricks.ActiveForm.Text = t;
                    break;
                }
            }
            if (!WallRD.Checked)
                WallRD.Checked = true;
            Drawbricks();
        }

        private void ColourBtn_Click(object sender, EventArgs e)
        {
            colorDialog1.FullOpen = true;
            colorDialog1.Color = brickcolor;
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                brickcolor = colorDialog1.Color;
                ColourTB.BackColor = brickcolor;
                ColourTB.Invalidate();
                Drawbricks();
            }
        }

        private void AltBtn_Click(object sender, EventArgs e)
        {
            colorDialog1.FullOpen = true;
            colorDialog1.Color = altbrickcolor;
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                altbrickcolor = colorDialog1.Color;
                AltColourTB.BackColor = altbrickcolor;
                AltColourTB.Invalidate();
                Drawbricks();
            }
        }

        private void MortarTB_Click(object sender, EventArgs e)
        {
            colorDialog1.FullOpen = true;
            colorDialog1.Color = mortarcolor;
            if (colorDialog1.ShowDialog() == DialogResult.OK)
            {
                mortarcolor = colorDialog1.Color;
                MortarTB.BackColor = mortarcolor;
                MortarTB.Invalidate();
                Drawbricks();
            }
        }

        private void CopyBtn_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
            Clipboard.SetImage(page);
        }

        private void TryBtn_Click(object sender, EventArgs e)
        {
            bond = PatternTB.Text;
            Drawbricks();
        }

        private void PrintBtn_Click(object sender, EventArgs e)
        {
            printDialog1.Document = printDocument1;
            printDocument1.DefaultPageSettings.Landscape = true;
            printDocument1.OriginAtMargins = false;
            if (printDialog1.ShowDialog() == DialogResult.OK)
            {
                printDocument1.Print();
            }
        }

        private void printDocument1_PrintPage(object sender, PrintPageEventArgs e)
        {
            float scale = (float)Convert.ToDouble(ScaleTB.Text) * 4; // required dpi
            page.SetResolution(scale, scale);
            e.Graphics.DrawImage(page, 0, 0);   
        }

        private void hScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            pageorgx = e.NewValue * (pagewidth - panel1.Width) / 100;
            panel1.Invalidate();
        }

        private void vScrollBar1_Scroll(object sender, ScrollEventArgs e)
        {
            pageorgy = e.NewValue * (pageheight - panel1.Height) / 100;
            panel1.Invalidate();
        }

     }
}
